#define BOOST_TEST_MODULE const_string test
#include <boost/test/unit_test.hpp>
#include <boost/bind.hpp>
#include <boost/mpl/list.hpp>
#include <boost/mpl/for_each.hpp>

BOOST_AUTO_TEST_CASE(BOOST_CHECK_Pass)
{
	BOOST_CHECK(true);
}

BOOST_AUTO_TEST_CASE(BOOST_WARN_Pass)
{
	BOOST_WARN(true);
}

BOOST_AUTO_TEST_CASE(BOOST_REQUIRE_Pass)
{
	BOOST_REQUIRE(true);
}

BOOST_AUTO_TEST_CASE(BOOST_CHECK_MESSAGE_Pass)
{
	BOOST_CHECK_MESSAGE(true, "text" << 1);
}


// BOOST_CHECK_EQUAL

BOOST_AUTO_TEST_CASE(BOOST_CHECK_EQUAL_Pass)
{
	BOOST_CHECK_EQUAL(1, 1);
}

// BOOST_CHECK_NE

BOOST_AUTO_TEST_CASE(BOOST_CHECK_NE_Pass)
{
	BOOST_CHECK_NE(1, 2);
}


// BOOST_CHECK_LT

BOOST_AUTO_TEST_CASE(BOOST_CHECK_LT_Pass)
{
	BOOST_CHECK_LT(1, 2);
}


// BOOST_CHECK_SMALL

BOOST_AUTO_TEST_CASE(BOOST_CHECK_SMALL_Pass)
{
	BOOST_CHECK_SMALL(0.008, 0.01);
}


// BOOST_CHECK_CLOSE

BOOST_AUTO_TEST_CASE(BOOST_CHECK_CLOSE_Pass)
{
	BOOST_CHECK_CLOSE(10.0, 10.05, 1.0);
}


// BOOST_CHECK_CLOSE_FRACTION

BOOST_AUTO_TEST_CASE(BOOST_CHECK_CLOSE_FRACTION_Pass)
{
	BOOST_CHECK_CLOSE_FRACTION(10.0, 10.05, 0.01);
}


// BOOST_CHECK_BITWISE_EQUAL

BOOST_AUTO_TEST_CASE(BOOST_CHECK_BITWISE_EQUAL_Pass)
{
	BOOST_CHECK_BITWISE_EQUAL(27, 27);
}

// BOOST_CHECK_EQUAL_COLLECTIONS

int arr1[3] = { 2, 3, 7 };
int arr2[3] = { 2, 3, 7 };
int arr3[3] = { 2, 4, 7 };

BOOST_AUTO_TEST_CASE(BOOST_CHECK_EQUAL_COLLECTIONS_Pass)
{
	BOOST_CHECK_EQUAL_COLLECTIONS(arr1, arr1+3, arr2, arr2+3);
}


bool is_even(int n)
{
	return n%2==0;
}

// BOOST_CHECK_PREDICATE with 1 argument

BOOST_AUTO_TEST_CASE(BOOST_CHECK_PREDICATE_1ARG_Pass)
{
	BOOST_CHECK_PREDICATE(is_even, (2));
}


bool sum_is_10(int a, int b, int c, int d, int e)
{
	return a+b+c+d+e==10;
}

// Tests for BOOST_CHECK_PREDICATE with 5 arguments

BOOST_AUTO_TEST_CASE(BOOST_CHECK_PREDICATE_5ARGS_Pass)
{
	BOOST_CHECK_PREDICATE(sum_is_10, (1)(2)(2)(3)(2));
}



// Tests for BOOST_CHECK_THROW

BOOST_AUTO_TEST_CASE(BOOST_CHECK_THROW_Pass)
{
	BOOST_CHECK_THROW(throw 2, int);
}


// BOOST_CHECK_NO_THROW

BOOST_AUTO_TEST_CASE(BOOST_CHECK_NO_THROW_Pass)
{
	BOOST_CHECK_NO_THROW(int a=2);
}


// BOOST_CHECK_EXCEPTION

bool Is2(int n)
{
	return n==2;
}

BOOST_AUTO_TEST_CASE(BOOST_CHECK_EXCEPTION_Pass)
{
	BOOST_CHECK_EXCEPTION(throw 2, int, Is2);
}

// tests for BOOST_IS_DEFINED

#define EMPTY_SYMBOL
#define SYMBOL_NO_ARGS 1
#define SYMBOL_ARGS(a,b) a b

BOOST_AUTO_TEST_CASE(EmptySymbol_Ok)
{
	BOOST_CHECK(BOOST_IS_DEFINED(EMPTY_SYMBOL));
}

BOOST_AUTO_TEST_CASE(SymbolNoArgs_Ok)
{
	BOOST_CHECK(BOOST_IS_DEFINED(SYMBOL_NO_ARGS));
}

BOOST_AUTO_TEST_CASE(SymbolWithArgs_Ok)
{
	BOOST_CHECK(BOOST_IS_DEFINED(SYMBOL_ARGS(1,2)));
}


// template tests

typedef boost::mpl::list<int,float> test_types;

int overloaded(int )
{
	return 1;
}

int overloaded(float )
{
	return 1;
}

BOOST_AUTO_TEST_CASE_TEMPLATE(AutoTemplateTest, T, test_types)
{
	BOOST_CHECK(overloaded(T(0))==1);
}


// BOOST_FIXTURE_TEST_CASE

struct Fixture
{
	Fixture(): n(5) {}
	int n;
};

BOOST_FIXTURE_TEST_CASE( TestWithFixture, Fixture)
{
    BOOST_CHECK(n==5);
}

// BOOST_AUTO_TEST_SUITE

BOOST_AUTO_TEST_SUITE(Suite)

	BOOST_AUTO_TEST_CASE( TestInSuite )
	{
		BOOST_CHECK(true);
	}

	BOOST_FIXTURE_TEST_CASE( TestWithFixtureInSuite, Fixture)
	{
		BOOST_CHECK(n==5);
	}

BOOST_AUTO_TEST_SUITE_END()


// BOOST_TEST_CHECKPOINT

BOOST_AUTO_TEST_CASE(BOOST_TEST_CHECKPOINT)
{
	int i=2;
	BOOST_TEST_CHECKPOINT("i=" << i);
	BOOST_CHECK(i==2);
}

// BOOST_AUTO_TEST_CASE_EXPECTED_FAILURES
BOOST_AUTO_TEST_CASE_EXPECTED_FAILURES(TestWithFailures_Ignored, 1)

BOOST_AUTO_TEST_CASE(TestWithFailures_Ignored)
{
	BOOST_CHECK(false);
}

// Global fixture

struct GlobalFixture
{
	GlobalFixture()  { n = 1; }
	~GlobalFixture() { n = 2; }
	static int n;
};

int GlobalFixture::n = 0;

BOOST_GLOBAL_FIXTURE(GlobalFixture);

BOOST_AUTO_TEST_CASE(Test_GlobalFixture)
{
	BOOST_CHECK_EQUAL(GlobalFixture::n, 1);
}
