#include "CppUnitTests.h"
#include <cppunit/config/SourcePrefix.h>
#include <cppunit/Exception.h>

//////////////////////////////////////////////////////////////
// SimpleTestWithName

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(SimpleTestWithName, "not used");

void SimpleTestWithName::testOk()
{
	CPPUNIT_ASSERT(true);
}



////////////////////////////////////////////////////////////
// Tests with fixture:

CPPUNIT_TEST_SUITE_REGISTRATION( FixtureTest );

void FixtureTest::setUp()
{
	setUpDone = true;
}

void FixtureTest::tearDown()
{
	setUpDone = false;
}

void FixtureTest::testOK()
{
	CPPUNIT_ASSERT(true);
}

void FixtureTest::testSetupCalled()
{
	CPPUNIT_ASSERT(setUpDone);
}


// Tests for exceptions:

CPPUNIT_TEST_SUITE_REGISTRATION( Exception );

void Exception::testThrowOk()
{
	throw 1;
}

// Tests for custom exceptions:

CPPUNIT_TEST_SUITE_REGISTRATION( CustomExceptionCheckerTest );

void CustomExceptionCheckerTest::testThrowOk()
{
	throw 1;
}


// Tests for CPPUNIT_TEST_SUB_SUITE

CPPUNIT_TEST_SUITE_REGISTRATION( FixtureBase );

void FixtureBase::setUp()
{
	a = 1;
}

void FixtureBase::tearDown()
{
}

void FixtureBase::testA()
{
	CPPUNIT_ASSERT(a==1);
}



CPPUNIT_TEST_SUITE_REGISTRATION( FixtureDerived );

void FixtureDerived::setUp()
{
	b = 2;
}

void FixtureDerived::tearDown()
{
}

void FixtureDerived::testB()
{
	CPPUNIT_ASSERT(a==1);
	CPPUNIT_ASSERT(b==2);
}



// Tests for abstract testsuite

void AbstractBase::setUp()
{
	a = 1;
}

void AbstractBase::tearDown()
{
}

void AbstractBase::testA()
{
	CPPUNIT_ASSERT_EQUAL(a,2);
}



CPPUNIT_TEST_SUITE_REGISTRATION( DerivedFromAbstract );

void DerivedFromAbstract::setUp()
{
	a = 2;
	b = 3;
}

void DerivedFromAbstract::tearDown()
{
}

void DerivedFromAbstract::testB()
{
	CPPUNIT_ASSERT(b==3);
}



// Tests for template test case:

// add a test with a shorter type to see it fail
CPPUNIT_TEST_SUITE_REGISTRATION( TemplateTest<float> );
CPPUNIT_TEST_SUITE_REGISTRATION( TemplateTest<int> );

template <typename T>
void TemplateTest<T>::testSizeOf()
{
	CPPUNIT_ASSERT(sizeof(T)==4);
}



// Test for custom TestCallers

CPPUNIT_TEST_SUITE_REGISTRATION( TestCaller );

void TestCaller::testCallerIsUsed()
{
	CPPUNIT_ASSERT(CustomTestCaller<TestCaller>::active);
}



// Test for custom TestCallers

CPPUNIT_TEST_SUITE_REGISTRATION( ThrowException );

void ThrowException::throwException()
{
	// uncomment to see this fail
	//throw CPPUNIT_NS::Exception(CPPUNIT_NS::Message("message"), CPPUNIT_SOURCELINE());
}



// Test for custom protectors

bool CustomProtector1::active = false;
bool CustomProtector2::active = false;

CPPUNIT_TEST_SUITE_REGISTRATION( CustomProtectorTest );

// in this test no protectors are used:
void CustomProtectorTest::testNotActive()
{
	CPPUNIT_ASSERT(!CustomProtector1::active);
	CPPUNIT_ASSERT(!CustomProtector2::active);
}

// in this test the first protector is used:
void CustomProtectorTest::testFirstActive()
{
	CPPUNIT_ASSERT(CustomProtector1::active);
	CPPUNIT_ASSERT(!CustomProtector2::active);
}

// in this test the first protector is also used, but with a guard:
void CustomProtectorTest::testFirstActiveWithGuard()
{
	CPPUNIT_ASSERT(CustomProtector1::active);
	CPPUNIT_ASSERT(!CustomProtector2::active);
}

// in this test both protectors are used:
void CustomProtectorTest::testBothActive()
{
	CPPUNIT_ASSERT(CustomProtector1::active);
	CPPUNIT_ASSERT(CustomProtector2::active);
}


// Test for custom TestCaller that call addError and addFailure

template <class T>
void CustomTestCallerWithFailure<T>::run(CPPUNIT_NS::TestResult *result)
{
	Parent::run(result);
	CPPUNIT_NS::Exception exc(CPPUNIT_NS::Message("message"), CPPUNIT_SOURCELINE());
	result->addFailure(this, &exc);
}

template <class T>
void CustomTestCallerWithException<T>::run(CPPUNIT_NS::TestResult *result)
{
	Parent::run(result);
	CPPUNIT_NS::Exception exc(CPPUNIT_NS::Message("exc", "message"), CPPUNIT_SOURCELINE());
	result->addError(this, &exc);
}



// Test for CPPUNIT_TEST_FAIL

CPPUNIT_TEST_SUITE_REGISTRATION( FailTest );

void FailTest::testFail_Ok()
{
	CPPUNIT_ASSERT(false); // ok because it is supposed to fail
}



// Tests for CPPUNIT_TEST_SUITE_ADD_CUSTOM_TESTS


CPPUNIT_TEST_SUITE_REGISTRATION( AddCustomTests );

void AddCustomTests::test1()
{
	CPPUNIT_ASSERT(true);
}

void AddCustomTests::test2()
{
	CPPUNIT_ASSERT(true);
}



// Tests for properties in TestSuiteBuilderContextBase
/*
class DummyTestFactory: public CPPUNIT_NS::TestFixtureFactory
{
public:
	virtual CPPUNIT_NS::TestFixture* makeFixture()
	{
		return 0;
	}
} dummyTestFactory;

CPPUNIT_TEST_SUITE_REGISTRATION( Properties );

Properties::Properties()
	: m_testNamer("Properties")
{}

void Properties::testEmptyProperties()
{
	CPPUNIT_NS::TestSuiteBuilderContextBase testSuiteBuilderContext("", m_testNamer, dummyTestFactory);
	CPPUNIT_ASSERT_EQUAL("", testSuiteBuilderContext.getStringProperty("abc"));
}

void Properties::testOneProperty()
{
	CPPUNIT_NS::TestSuiteBuilderContextBase testSuiteBuilderContext("", m_testNamer, dummyTestFactory);
	testSuiteBuilderContext.addProperty("abc", "xyz");
	CPPUNIT_ASSERT_EQUAL("xyz", testSuiteBuilderContext.getStringProperty("abc"));
}

void Properties::testTwoProperties()
{
	CPPUNIT_NS::TestSuiteBuilderContextBase testSuiteBuilderContext("", m_testNamer, dummyTestFactory);
	testSuiteBuilderContext.addProperty("abc", "xyz");
	testSuiteBuilderContext.addProperty("def", "123");
	CPPUNIT_ASSERT_EQUAL("xyz", testSuiteBuilderContext.getStringProperty("abc"));
	CPPUNIT_ASSERT_EQUAL("123", testSuiteBuilderContext.getStringProperty("def"));
}

void Properties::testOverrideProperty()
{
	CPPUNIT_NS::TestSuiteBuilderContextBase testSuiteBuilderContext("", m_testNamer, dummyTestFactory);
	testSuiteBuilderContext.addProperty("abc", "xyz");
	testSuiteBuilderContext.addProperty("abc", "123");
	CPPUNIT_ASSERT_EQUAL("123", testSuiteBuilderContext.getStringProperty("abc"));
}

void Properties::testOverrideTwoProperties()
{
	CPPUNIT_NS::TestSuiteBuilderContextBase testSuiteBuilderContext("", m_testNamer, dummyTestFactory);
	testSuiteBuilderContext.addProperty("abc", "xyz");
	testSuiteBuilderContext.addProperty("def", "123");
	testSuiteBuilderContext.addProperty("abc", "pqr");
	testSuiteBuilderContext.addProperty("def", "456");
	CPPUNIT_ASSERT_EQUAL("pqr", testSuiteBuilderContext.getStringProperty("abc"));
	CPPUNIT_ASSERT_EQUAL("456", testSuiteBuilderContext.getStringProperty("def"));
}
*/


// Tests for CPPUNIT_TEST_SUITE_PROPERTY


CPPUNIT_TEST_SUITE_REGISTRATION( UseProperties );

void UseProperties::test1()
{
	CPPUNIT_ASSERT_EQUAL(m_param, std::string("abc"));
}



// Tests for TestDecorator

int MyTestDecorator::beforeCalled = 0;
int MyTestDecorator::afterCalled = 0;

CPPUNIT_TEST_SUITE_REGISTRATION( TestDecorator );

void TestDecorator::test1()
{
	CPPUNIT_ASSERT_EQUAL(MyTestDecorator::beforeCalled, 1);
	CPPUNIT_ASSERT_EQUAL(MyTestDecorator::afterCalled, 0);
}


void TestDecorator::test2()
{
	CPPUNIT_ASSERT_EQUAL(MyTestDecorator::beforeCalled, 2);
	CPPUNIT_ASSERT_EQUAL(MyTestDecorator::afterCalled, 1);
}



// Tests for TestCaseDecorator

int MyTestCaseDecorator::setUpCalled = 0;
int MyTestCaseDecorator::runCalled = 0;
int MyTestCaseDecorator::tearDownCalled = 0;

int TestCaseDecorator::setUpCalled = 0;
int TestCaseDecorator::tearDownCalled = 0;

CPPUNIT_TEST_SUITE_REGISTRATION( TestCaseDecorator );

void TestCaseDecorator::test1()
{
	CPPUNIT_ASSERT_EQUAL(MyTestCaseDecorator::setUpCalled, 1);
	CPPUNIT_ASSERT_EQUAL(MyTestCaseDecorator::runCalled, 1);
	CPPUNIT_ASSERT_EQUAL(MyTestCaseDecorator::tearDownCalled, 0);

	CPPUNIT_ASSERT_EQUAL(setUpCalled, 1);
	CPPUNIT_ASSERT_EQUAL(tearDownCalled, 0);
}

void TestCaseDecorator::test2()
{
	CPPUNIT_ASSERT_EQUAL(MyTestCaseDecorator::setUpCalled, 2);
	CPPUNIT_ASSERT_EQUAL(MyTestCaseDecorator::runCalled, 2);
	CPPUNIT_ASSERT_EQUAL(MyTestCaseDecorator::tearDownCalled, 1);

	CPPUNIT_ASSERT_EQUAL(setUpCalled, 2);
	CPPUNIT_ASSERT_EQUAL(tearDownCalled, 1);
}



// Tests for RepeatedTest

CPPUNIT_TEST_SUITE_REGISTRATION( RepeatedTest );

void RepeatedTest::test1()
{
	CPPUNIT_ASSERT(true);
}

void RepeatedTest::test2()
{
	CPPUNIT_ASSERT(true);
}


// Tests for TestSetup

int MyTestSetUp::setUpCalled = 0;
int MyTestSetUp::tearDownCalled = 0;

CPPUNIT_TEST_SUITE_REGISTRATION( TestSetUp );

void TestSetUp::test1()
{
	CPPUNIT_ASSERT_EQUAL(MyTestSetUp::setUpCalled, 1);
	CPPUNIT_ASSERT_EQUAL(MyTestSetUp::tearDownCalled, 0);
}

void TestSetUp::test2()
{
	CPPUNIT_ASSERT_EQUAL(MyTestSetUp::setUpCalled, 2);
	CPPUNIT_ASSERT_EQUAL(MyTestSetUp::tearDownCalled, 1);
}


// tests for Orthodoxy:

// for Orthodox_Ok:

CPPUNIT_TEST_SUITE_REGISTRATION(Orthodoxy_Ok);


// for TestComposite

CPPUNIT_TEST_SUITE_REGISTRATION(CompositeTest);

void CompositeTest::test1()
{
	CPPUNIT_ASSERT(true);
}

void CompositeTest::test2()
{
	CPPUNIT_ASSERT(true);
}
