#include <cxxtest/TestSuite.h>

// Basic test

class TestSuite_Basic: public CxxTest::TestSuite
{
public:
    void testAddition(void)
    {
        TS_ASSERT(1+1 == 2);
    }
    void testAdditionWith_AssertEquals(void)
    {
        TS_ASSERT_EQUALS(1 + 1, 2);
    }
};

// Assert throw

class TestSuiteThrow: public CxxTest::TestSuite
{
public:
    void testThrowInt(void)
    {
        TS_ASSERT_THROWS(throw 1, int);
    }
    void testThrowsAnything(void)
    {
        TS_ASSERT_THROWS_ANYTHING(throw 1);
    }
};


// Test with fixture

class TestWithFixture: public CxxTest::TestSuite
{
    char *_buffer;

public:

    void setUp()
    {
        _buffer = new char[1024];
    }

    void tearDown()
    {
        delete [] _buffer;
    }

    void test_strcpy()
    {
        strcpy(_buffer, "Hello, world!");
        TS_ASSERT_EQUALS(_buffer[0], 'H');
        TS_ASSERT_EQUALS(_buffer[1], 'e');
    }
};


// Dynamic TestSuite

class DynamicTestSuite: public CxxTest::TestSuite
{
public:
	DynamicTestSuite(int n)
		: m_n(n)
	{}

	int m_n;

	static DynamicTestSuite* createSuite()
    {
	    return new DynamicTestSuite(7);
    }

    static void destroySuite( DynamicTestSuite* suite )
    {
		delete suite;
	}

    void testAddAndSubtract()
    {
        int a = 1;
		a += m_n;
		a -= m_n;
		TS_ASSERT_EQUALS(1, a);
    }
};


// value traits

class TwoInts
{
public:
	TwoInts(int n1, int n2)
		: m_n1(n1)
		, m_n2(n2)
	{}

	int m_n1, m_n2;

	bool operator==(const TwoInts &t) const
	{
		return m_n1==t.m_n1 && m_n2==t.m_n2;
	}
};

namespace CxxTest
{
	CXXTEST_TEMPLATE_INSTANTIATION
	class ValueTraits<TwoInts>
	{
		char _s[256];

	public:
		ValueTraits(const TwoInts &m)
		{
			sprintf( _s, "(%i,%i)", m.m_n1, m.m_n2);
		}
		const char *asString() const
		{
			return _s;
		}
	};
}

class TestSuite_TwoInts: public CxxTest::TestSuite
{
public:
    void testTwoInts(void)
    {
		TwoInts a(1,2), b(1,2);
		TS_ASSERT_EQUALS(a, b);
    }
};


// template value traits

template <typename T>
class TwoValues
{
public:
	TwoValues(T n1, T n2)
		: m_n1(n1)
		, m_n2(n2)
	{}

	T m_n1, m_n2;

	bool operator==(const TwoValues &t) const
	{
		return m_n1==t.m_n1 && m_n2==t.m_n2;
	}
};

namespace CxxTest
{
	template <typename T>
	class ValueTraits< TwoValues<T> >
	{
		std::string str;

	public:
		ValueTraits(const TwoValues<T> &m)
		{
			std::ostringstream strstr;
			strstr << "(" << m.m_n1 << "," << m.m_n2 << ")";
			str = strstr.str();
		}
		const char *asString() const
		{
			return str.c_str();
		}
	};
}

class TestSuite_TwoValues: public CxxTest::TestSuite
{
public:
    void testTwoInts(void)
    {
		TwoValues<float> a(1.1,2.5), b(1.1,2.5);
		TS_ASSERT_EQUALS(a, b);
    }
};

// Mock a function

#define CXXTEST_MOCK_TEST_SOURCE_FILE
#include "TimeMock.h"
#include "TimeUtils.h"

class MockTime : public T::Base_time
{
public:
    MockTime(int initial) : counter(initial) {}
    int counter;
    time_t time( time_t * ) { return counter++; }
};

class TestSuite_MockTime: public CxxTest::TestSuite
{
public:
    void testNotMidnight()
    {
		MockTime t(1);
        TS_ASSERT(!IsMidnight());
    }
    void testMidnight()
    {
		MockTime t(3600*24);
        TS_ASSERT(IsMidnight());
    }
};
