//  Copyright (C) 2006-2009 Q-Mentum.
//  See the file license.txt for limitations on usage of this code.

//  For more information about Q-Mentum, visit http://www.q-mentum.com

#include "uquonitest/Test.hpp"
#include "uquonitest/TestWrapper.hpp"
#include "richbool/richbool.hpp"
#include "richbool/containers.hpp"

#include "SumOfIntsInFiles.hpp"


// This testgroup demonstrates how UquoniTest can create temporary directories for you.

uqtTEST_GROUP(SumOfIntsInFiles)
{
	// Example of a test that begins with an empty directory,
	// and puts some files in it.
	uqtTEST(StartWithEmptyDir)
	{
		UquoniTest::TestDirectory dir;
		dir.MakeFile("files.txt", "file1.txt file2.txt");
		dir.MakeFile("file1.txt", "1");
		dir.MakeFile("file2.txt", "2");

		SumOfIntsInFiles sumOfIntsInFiles;
		sumOfIntsInFiles.Process(dir.ToString());

		uqtASSERT(rbEQUAL(3, sumOfIntsInFiles.GetSum()));

		std::string expectedLines[] = { "Nr of files: 2", "Sum: 3" };
		std::vector<std::string> actualLines(UquoniTest::FileLinesIterator(dir/"results.txt"), UquoniTest::FileLinesIterator());
		uqtASSERT(rbIN_ARRAY_CONTAINER(expectedLines, actualLines, RichBool::AllEqual<>()));
	}

	// This creates a predefined test directory that can be reused in tests.
	uqtTEST_DIRECTORY(OneFileWithTwoNumbers)
	{
		uqtDirectory.MakeFile("files.txt", "numbers.txt");
		uqtDirectory.MakeFile("numbers.txt", "1 3");
	}

	// This test uses the predefined directory above.
	uqtTEST(PredefinedDir)
	{
		UquoniTest::TestDirectory dir("OneFileWithTwoNumbers");

		SumOfIntsInFiles sumOfIntsInFiles;
		sumOfIntsInFiles.Process(dir.ToString());

		uqtASSERT(rbEQUAL(4, sumOfIntsInFiles.GetSum()));

		std::string expectedLines[] = { "Nr of files: 1", "Sum: 4" };
		std::vector<std::string> actualLines(UquoniTest::FileLinesIterator(dir/"results.txt"), UquoniTest::FileLinesIterator());
		uqtASSERT(rbIN_ARRAY_CONTAINER(expectedLines, actualLines, RichBool::AllEqual<>()));
	}
}
