//  Copyright (C) 2006-2009 Q-Mentum.
//  See the file license.txt for limitations on usage of this code.

//  For more information about Q-Mentum, visit http://www.q-mentum.com

#include "TestEncryption.hpp"
#include "Encryption.hpp"

// This is a demonstration of abstract tests.
// Here is the implementation of the abstract testgroup.

// See also the files TestEncryption.hpp, TestXOREncryption.cpp
// and TestJuliusCaesarEncryption.cpp.


uqtTEST_GROUP(Encryption)
{
	// This is the implementation of the abstract test EncryptDecryptABC.
	// For this we need to define a method with the same name in the fixture
	// class in TestEncryption.hpp, without arguments.

	uqtABSTRACT_TESTF(EncryptDecryptABC)
	{
		std::string encrypted = encryption->Encrypt("ABC");
		uqtASSERT(rbDIFF(encrypted, "ABC"));

		std::string decrypted = encryption->Decrypt(encrypted);
		uqtASSERT(rbEQUAL(decrypted, "ABC"));
	}


	// This is the implementation of the abstract parameterized test EncryptDecrypt.
	// For this we need to define a method with the same name in the fixture
	// class in TestEncryption.hpp, with a string as an argument

	uqtDEFINE_ABSTRACT_TESTF_1PARAM(EncryptDecrypt, std::string, str)
	{
		std::string encrypted = encryption->Encrypt(str);
		uqtASSERT(rbDIFF(encrypted, str));

		std::string decrypted = encryption->Decrypt(encrypted);
		uqtASSERT(rbEQUAL(decrypted, str));
	}

	uqtABSTRACT_TESTF_1PARAM(EncryptDecrypt, "a");
	uqtABSTRACT_TESTF_1PARAM(EncryptDecrypt, "ab");
	uqtABSTRACT_TESTF_1PARAM(EncryptDecrypt, "abcdef");
	uqtABSTRACT_TESTF_1PARAM(EncryptDecrypt, "\x01\xff");
}
