//  Copyright (C) 2006-2009 Q-Mentum.
//  See the file license.txt for limitations on usage of this code.

//  For more information about Q-Mentum, visit http://www.q-mentum.com

#include "uquonitest/Test.hpp"
#include "richbool/richbool.hpp"

#include "Interface.hpp"
#include "UseInterface.hpp"

class StubInterface: public Interface
{
public:
	UquoniTest::MethodMock1<int> m_mockFoo;
	virtual void Foo(int a)
	{
		m_mockFoo(a);
	}

	UquoniTest::MethodMock0<int> m_mockBar;
	virtual int Bar()
	{
		return m_mockBar();
	}
};

uqtTEST_GROUP(UseInterface)
{
	class uqtFixture
	{
	public:
		uqtFixture(): useInterface(&stubInterface)
		{}

		StubInterface stubInterface;
		UseInterface useInterface;
	};

	uqtTESTF(CallFoo_Once)
	{
		stubInterface.m_mockFoo.SetAction(
			uqtMOCK_ASSERT(RichBool::Equals<int>(0))+
			uqtCALLED(1)
		);

		useInterface.CallFoo();
	}

	uqtTESTF(CallFoo_Twice)
	{
		UquoniTest::ActionSequence1<int> actions;
		actions.Add(uqtMOCK_ASSERT(RichBool::Equals<int>(0)));
		actions.Add(uqtMOCK_ASSERT(RichBool::Equals<int>(1)));

		stubInterface.m_mockFoo.SetAction(actions);

		useInterface.CallFoo();
		useInterface.CallFoo();
	}

	uqtTESTF(SumOf3Calls)
	{
		stubInterface.m_mockBar.SetAction(UquoniTest::Return(2));

		int sum = useInterface.GetSumOf3CallsToBar();
		uqtCASSERT(rbEQUAL(6, sum));
		uqtCASSERT(rbEQUAL(3, stubInterface.m_mockBar.GetNrCalls()));
	}

	uqtTESTF(CallFoo_ExceptionNotCaught)
	{
		stubInterface.m_mockFoo.SetAction(UquoniTest::Throw(Exception()));

		uqtASSERT_THROWS(useInterface.CallFoo(), Exception);
	}

	uqtTESTF(CallFooCatchException_ExceptionCaught)
	{
		stubInterface.m_mockFoo.SetAction(UquoniTest::Throw(Exception()));

		useInterface.CallFoo_CatchException();
	}
}
