//  Copyright (C) 2006-2009 Q-Mentum.
//  See the file license.txt for limitations on usage of this code.

//  For more information about Q-Mentum, visit http://www.q-mentum.com

#include "uquonitest/Test.hpp"
#include "richbool/richbool.hpp"

#include <vector>

// The tests in this group perform tests on the class std::vector.
// These tests demonstrate the use of template tests.

uqtTEST_GROUP(Vector)
{
	// definition of a template test:
	uqtDEFINE_TEST_1TMPL_1PARAM(Add1, typename, T, T, t1)
	{
		std::vector<T> vec;
		vec.push_back(t1);

		uqtASSERT(rbEQUAL(vec.size(), 1));
		uqtCASSERT(rbEQUAL(vec[0], t1));
	}

	// usage of the template test:
	uqtTEST_1TMPL_1PARAM(Add1, int, 1);
	uqtTEST_1TMPL_1PARAM(Add1, double, 1.0);



	// definition of a template test:
	uqtDEFINE_TEST_1TMPL_3PARAM(Add3, typename, T, T, t1, T, t2, T, t3)
	{
		std::vector<T> vec;
		vec.push_back(t1);
		vec.push_back(t2);
		vec.push_back(t3);

		uqtASSERT(rbEQUAL(vec.size(), 3));
		uqtCASSERT(rbEQUAL(vec[0], t1));
		uqtCASSERT(rbEQUAL(vec[1], t2));
		uqtCASSERT(rbEQUAL(vec[2], t3));
	}

	// usage of the template test:
	uqtTEST_1TMPL_3PARAM(Add3, int, 1, 2, 3);
	uqtTEST_1TMPL_3PARAM(Add3, double, 1.0, 2.0, 3.0);



	// definition of a template test:
	uqtDEFINE_TEST_1TMPL_3PARAM(Add3Remove2nd, typename, T, T, t1, T, t2, T, t3)
	{
		std::vector<T> vec;
		vec.push_back(t1);
		vec.push_back(t2);
		vec.push_back(t3);
		vec.erase(vec.begin()+1);

		uqtASSERT(rbEQUAL(vec.size(), 2));
		uqtCASSERT(rbEQUAL(vec[0], t1));
		uqtCASSERT(rbEQUAL(vec[1], t3));
	}

	// usage of the template test:
	uqtTEST_1TMPL_3PARAM(Add3Remove2nd, int, 1, 2, 3);
	uqtTEST_1TMPL_3PARAM(Add3Remove2nd, double, 1.0, 2.0, 3.0);
}
