// gTest.cpp : Defines the entry point for the console application.
//

#include "gtest/gtest.h"

// Basic test
TEST(Example, A)
{
	EXPECT_EQ(1+1,2);
}

// Disabling a test
TEST(DISABLED_Example, A)
{
	EXPECT_EQ(1+1,3); // test disabled until we understand what is wrong here
}


///////////////////////////////////////////////////////////
// using a fixture

class Fixture: public ::testing::Test
{
public:
	Fixture()
		: a(1)
		, b(2)
	{}

	int a, b;
};

TEST_F(Fixture, a)
{
	EXPECT_EQ(a,1);
}

TEST_F(Fixture, b)
{
	EXPECT_EQ(b,2);
}

////////////////////////////////////////////////////////////
// using a predicate

bool Is1(int n)
{
	return n==1;
}

TEST(WithPredicate, A)
{
	EXPECT_PRED1(Is1, 1);
}

////////////////////////////////////////////////////////////
// using function that returns an assertion result

::testing::AssertionResult IsEven(int n) {
  if ((n % 2) == 0)
    return ::testing::AssertionSuccess();
  else
    return ::testing::AssertionFailure() << n << " is odd";
}

::testing::AssertionResult IsEven_Format(const char *expr, int n) {
  if ((n % 2) == 0)
    return ::testing::AssertionSuccess();
  else
    return ::testing::AssertionFailure() << expr << "=" << n << " is odd";
}

TEST(WithPredicate, B)
{
	int n = 2;
	EXPECT_TRUE(IsEven(n));
	EXPECT_PRED_FORMAT1(IsEven_Format, n);
}

////////////////////////////////////////////////////////////
// type assertions

TEST(TypeAssertion, A)
{
	// causes compile error if they are different
	::testing::StaticAssertTypeEq<std::string, std::string>();
}

////////////////////////////////////////////////////////////
// death tests

TEST(MyDeathTest, A)
{
	int *p = 0;
	EXPECT_DEATH(*p = 1, ""); 
}

TEST(MyDeathTest, B)
{
	EXPECT_EXIT(exit(1), Is1, ""); 
}

////////////////////////////////////////////////////////////
// Global Set-Up and Tear-Down

int globalInt = 0;

class MyEnvironment: public ::testing::Environment
{
public:
	 MyEnvironment()
	 {
		 AddGlobalTestEnvironment(this);
	 }
	virtual void SetUp()
	{
	  globalInt = 1;
	}
	virtual void TearDown()
	{
	  globalInt = -1;
	}
} *myEnvironment = new MyEnvironment;

TEST(GlobalInt, A)
{
	EXPECT_EQ(1, globalInt); 
}


////////////////////////////////////////////////////////////
// Value-Parameterized Tests

class ParamTest : public ::testing::TestWithParam<int>
{
	int n;
};

TEST_P(ParamTest, DoesBlah)
{
	EXPECT_LT(GetParam(), 5);
}

INSTANTIATE_TEST_CASE_P(InstantiationWithValues,
                        ParamTest,
                        ::testing::Values(1, 2, 3)
                        );

INSTANTIATE_TEST_CASE_P(InstantiationWithRange,
                        ParamTest,
                        ::testing::Range(1, 5)
                        );


// Typed Tests

template <typename T>
class TypedTest: public ::testing::Test
{
public:
	typedef T T_;
	T value;
};

typedef ::testing::Types<int, unsigned int, float, char> MyTypes;
TYPED_TEST_CASE(TypedTest, MyTypes);

TYPED_TEST(TypedTest, SizeIs4)
{
	EXPECT_EQ(4, sizeof(T_));
}


// Type-Parameterized Tests

template <typename T>
class TypeParameterizedTest : public ::testing::Test
{
public:
	typedef T T_;
	T value;
};

TYPED_TEST_CASE_P(TypeParameterizedTest);

TYPED_TEST_P(TypeParameterizedTest, SizeIs4)
{
	EXPECT_EQ(4, sizeof(T_));
}

REGISTER_TYPED_TEST_CASE_P(TypeParameterizedTest,
                           SizeIs4);

INSTANTIATE_TYPED_TEST_CASE_P(TypeParameterizedTestInstantiation, TypeParameterizedTest, MyTypes);
