//  Copyright (C) 2006-2009 Q-Mentum.
//  See the file license.txt for limitations on usage of this code.

//  For more information about Q-Mentum, visit http://www.q-mentum.com

#include "uquonitest/Test.hpp"
#include "richbool/richbool.hpp"

#include "DNA.hpp"

// This group demonstrates the use of parameterized tests,
// and testing whether an assertion fails in domain code.

uqtTEST_GROUP(DNA)
{
	// definition of a parameterized test:
	uqtDEFINE_TEST_2PARAM(Complement,
		std::string, orig,
		std::string, complement)
	{
		Complement(orig);
		uqtASSERT(rbEQUAL(orig, complement));
	}

	// usage of the parameterized test:
	uqtTEST_2PARAM(Complement, "a", "t");

	uqtTEST_2PARAM(Complement, "ac", "tg");



	// definition of a parameterized test:
	uqtDEFINE_TEST_2PARAM(ComplementReverse,
		std::string, orig,
		std::string, complementReverse)
	{
		ComplementReverse(orig);
		uqtASSERT(rbEQUAL(orig, complementReverse));
	}

	// usage of the parameterized test:
	uqtTEST_2PARAM(ComplementReverse, "a", "t");

	uqtTEST_2PARAM(ComplementReverse, "ac", "gt");




	// Now test if Complement makes an assertion fail when it has to,
	// namely when the sequence contains an invalid character (other than acgt):
	
	uqtTEST(OneAssertion)
	{
		std::string str = "agbct";
#ifndef NDEBUG
		uqtEXPECT_MOD_ASSERT(Complement(str));
#else
		// we use the default behaviour of ModAssert,
		// where assertions are disabled if NDEBUG is defined,
		// so test that it does NOT have failing assertions:
		Complement(str);
#endif
	}

	uqtTEST(TwoAssertions)
	{
		std::string str = "agbczt";
#ifndef NDEBUG
		uqtEXPECT_MOD_ASSERT(uqtEXPECT_MOD_ASSERT(Complement(str)));
#else
		Complement(str);
#endif
	}
}
